defmodule ClickHouse.Interface do
  @moduledoc """
  A behaviour to implement a network interface with a ClickHouse server.
  """

  @typedoc """
  The ClickHouse network interface.
  """
  @type t :: module()

  @doc """
  A callback to start the network interface.
  """
  @callback start_link(keyword()) :: GenServer.on_start()

  @doc """
  A callback to execute a query with the network interface.
  """
  @callback execute(
              client :: ClickHouse.Client.t(),
              query :: ClickHouse.Query.t(),
              opts :: keyword()
            ) ::
              {:ok, ClickHouse.Result.t()} | {:error, ClickHouse.error()}
end
